@@ -90,7 +90,7 @@ has_percent0A_suffix (const char *string)
 
    INFO gets displayed as part of a generic string.  However if the
    first character of INFO is a vertical bar all up to the next
-   verical bar are considered flags and only everything after the
+   vertical bar are considered flags and only everything after the
    second vertical bar gets displayed as the full prompt.
 
    Flags:
@@ -377,10 +377,10 @@ divert_pksign (ctrl_t ctrl, const unsigned char *grip,
 }
 
 
-/* Decrypt the value given asn an S-expression in CIPHER using the
+/* Decrypt the value given as an s-expression in CIPHER using the
    key identified by SHADOW_INFO and return the plaintext in an
    allocated buffer in R_BUF.  The padding information is stored at
-   R_PADDING with -1 for not known.  */
+   R_PADDING with -1 for not known, when it's not NULL.  */
 int
 divert_pkdecrypt (ctrl_t ctrl,
                   const unsigned char *grip,
@@ -399,7 +399,8 @@ divert_pkdecrypt (ctrl_t ctrl,
 
   bin2hex (grip, 20, hexgrip);
 
-  *r_padding = -1;
+  if (r_padding)
+    *r_padding = -1;
   s = cipher;
   if (*s != '(')
     return gpg_error (GPG_ERR_INV_SEXP);
@@ -467,7 +468,20 @@ divert_pkdecrypt (ctrl_t ctrl,
       n = snext (&s);
     }
   else
-    return gpg_error (GPG_ERR_UNSUPPORTED_ALGORITHM);
+    {
+      if (opt.verbose)
+        {
+          if (smatch (&s, n, "elg"))
+            log_info ("unknown algorithm is \"elg\"\n");
+          else if (smatch (&s, n, "dsa"))
+            log_info ("unknown algorithm is \"dsa\"\n");
+          else if (smatch (&s, n, "kyber"))
+            log_info ("unknown algorithm is \"kyber\"\n");
+          else
+            log_printhex (s, n, "unknown algorithm is");
+        }
+      return gpg_error (GPG_ERR_UNSUPPORTED_ALGORITHM);
+    }
 
   if (!n)
     return gpg_error (GPG_ERR_UNKNOWN_SEXP);
@@ -485,6 +499,34 @@ divert_pkdecrypt (ctrl_t ctrl,
   return rc;
 }
 
+gpg_error_t
+agent_card_ecc_kem (ctrl_t ctrl, const unsigned char *ecc_ct,
+                    size_t ecc_point_len, unsigned char *ecc_ecdh)
+{
+  gpg_error_t err = 0;
+  char *ecdh = NULL;
+  size_t len;
+  int rc;
+
+  rc = agent_card_pkdecrypt (ctrl, ctrl->keygrip, getpin_cb, ctrl, NULL,
+                             ecc_ct, ecc_point_len, &ecdh, &len, NULL);
+  if (rc)
+    return rc;
+
+  if (len != ecc_point_len)
+    {
+      if (opt.verbose)
+        log_info ("%s: ECC result length invalid (%zu != %zu)\n",
+                  __func__, len, ecc_point_len);
+      return gpg_error (GPG_ERR_INV_DATA);
+    }
+  else
+    memcpy (ecc_ecdh, ecdh, len);
+
+  xfree (ecdh);
+  return err;
+}
+
 
 gpg_error_t
 divert_writekey (ctrl_t ctrl, int force, const char *serialno,
