@@ -34,6 +34,100 @@
 # include "ldap-parse-uri.h"
 #endif
 
+
+/* Parse an URI and store it in a new parsed URI item object which is
+ * returned at R_PARSEDURI (with its next set to NULL).  On error an
+ * error code is returned an NULL stored at R_PARSEDITEM.  */
+gpg_error_t
+ks_action_parse_uri (const char *uri, uri_item_t *r_parseduri)
+{
+  gpg_error_t err;
+  uri_item_t item;
+  char *tmpstr = NULL;
+#if USE_LDAP
+  const char *s;
+#endif
+
+  *r_parseduri = NULL;
+
+  if (!uri)
+    return gpg_error (GPG_ERR_INV_URI);
+
+  item = xtrymalloc (sizeof *item + strlen (uri));
+  if (!item)
+    return gpg_error_from_syserror ();
+
+  item->next = NULL;
+  item->parsed_uri = NULL;
+  strcpy (item->uri, uri);
+
+#if USE_LDAP
+  if (!strncmp (uri, "ldap:", 5) && !(uri[5] == '/' && uri[6] == '/'))
+    {
+      /* Special ldap scheme given.  This differs from a valid ldap
+       * scheme in that no double slash follows.  We use
+       * http_parse_uri to put it as opaque value into parsed_uri.  */
+      tmpstr = strconcat ("opaque:", uri+5, NULL);
+      if (!tmpstr)
+        err = gpg_error_from_syserror ();
+      else
+        err = http_parse_uri (&item->parsed_uri, tmpstr, 0);
+    }
+  else if ((s=strchr (uri, ':')) && !(s[1] == '/' && s[2] == '/'))
+    {
+      /* No valid scheme given.  We use http_parse_uri to put the
+       * string as opaque value into parsed_uri.  */
+      tmpstr = strconcat ("opaque:", uri, NULL);
+      if (!tmpstr)
+        err = gpg_error_from_syserror ();
+      else
+        err = http_parse_uri (&item->parsed_uri, tmpstr, 0);
+    }
+  else if (ldap_uri_p (uri))
+    {
+      int fixup = 0;
+      /* Fixme: We should get rid of that parser and replace it with
+       * our generic (http) URI parser.  */
+
+      /* If no port has been specified and the scheme ist ldaps we use
+       * our idea of the default port because the standard LDAP URL
+       * parser would use 636 here.  This is because we redefined
+       * ldaps to mean starttls.  */
+#ifdef HAVE_W32_SYSTEM
+      if (!strcmp (uri, "ldap:///"))
+          fixup = 1;
+      else
+#endif
+        if (!http_parse_uri (&item->parsed_uri,uri,HTTP_PARSE_NO_SCHEME_CHECK))
+        {
+          if (!item->parsed_uri->port
+              && !strcmp (item->parsed_uri->scheme, "ldaps"))
+            fixup = 2;
+          http_release_parsed_uri (item->parsed_uri);
+          item->parsed_uri = NULL;
+        }
+
+      err = ldap_parse_uri (&item->parsed_uri, uri);
+      if (!err && fixup == 1)
+        item->parsed_uri->ad_current = 1;
+      else if (!err && fixup == 2)
+        item->parsed_uri->port = 389;
+    }
+  else
+#endif /* USE_LDAP */
+    {
+      err = http_parse_uri (&item->parsed_uri, uri, HTTP_PARSE_NO_SCHEME_CHECK);
+    }
+
+  xfree (tmpstr);
+  if (err)
+    xfree (item);
+  else
+    *r_parseduri = item;
+  return err;
+}
+
+
 /* Called by the engine's help functions to print the actual help.  */
 gpg_error_t
 ks_print_help (ctrl_t ctrl, const char *text)
@@ -67,6 +161,10 @@ ks_action_help (ctrl_t ctrl, const char *url)
 {
   gpg_error_t err;
   parsed_uri_t parsed_uri;  /* The broken down URI.  */
+#if USE_LDAP
+  char *tmpstr;
+  const char *s;
+#endif
 
   if (!url || !*url)
     {
@@ -76,12 +174,39 @@ ks_action_help (ctrl_t ctrl, const char *url)
   else
     {
 #if USE_LDAP
-      if (ldap_uri_p (url))
+      if (!strncmp (url, "ldap:", 5) && !(url[5] == '/' && url[6] == '/'))
+        {
+          /* Special ldap scheme given.  This differs from a valid
+           * ldap scheme in that no double slash follows.  Use
+           * http_parse_uri to put it as opaque value into parsed_uri.  */
+          tmpstr = strconcat ("opaque:", url+5, NULL);
+          if (!tmpstr)
+            err = gpg_error_from_syserror ();
+          else
+            {
+              err = http_parse_uri (&parsed_uri, tmpstr, 0);
+              xfree (tmpstr);
+            }
+        }
+      else if ((s=strchr (url, ':')) && !(s[1] == '/' && s[2] == '/'))
+        {
+          /* No scheme given.  Use http_parse_uri to put the string as
+           * opaque value into parsed_uri.  */
+          tmpstr = strconcat ("opaque:", url, NULL);
+          if (!tmpstr)
+            err = gpg_error_from_syserror ();
+          else
+            {
+              err = http_parse_uri (&parsed_uri, tmpstr, 0);
+              xfree (tmpstr);
+            }
+        }
+      else if (ldap_uri_p (url))
 	err = ldap_parse_uri (&parsed_uri, url);
       else
 #endif
 	{
-	  err = http_parse_uri (&parsed_uri, url, 1);
+	  err = http_parse_uri (&parsed_uri, url, HTTP_PARSE_NO_SCHEME_CHECK);
 	}
 
       if (err)
@@ -164,9 +289,10 @@ ks_action_search (ctrl_t ctrl, uri_item_t keyservers,
       int is_ldap = 0;
       unsigned int http_status = 0;
 #if USE_LDAP
-      is_ldap = (strcmp (uri->parsed_uri->scheme, "ldap") == 0
-		 || strcmp (uri->parsed_uri->scheme, "ldaps") == 0
-		 || strcmp (uri->parsed_uri->scheme, "ldapi") == 0);
+      is_ldap = (!strcmp (uri->parsed_uri->scheme, "ldap")
+		 || !strcmp (uri->parsed_uri->scheme, "ldaps")
+		 || !strcmp (uri->parsed_uri->scheme, "ldapi")
+                 || uri->parsed_uri->opaque);
 #endif
       if (is_http || is_ldap)
         {
@@ -211,7 +337,8 @@ ks_action_search (ctrl_t ctrl, uri_item_t keyservers,
    keyservers and write the result to the provided output stream.  */
 gpg_error_t
 ks_action_get (ctrl_t ctrl, uri_item_t keyservers,
-	       strlist_t patterns, int ldap_only, estream_t outfp)
+	       strlist_t patterns, unsigned int ks_get_flags,
+               gnupg_isotime_t newer, estream_t outfp)
 {
   gpg_error_t err = 0;
   gpg_error_t first_err = 0;
@@ -238,13 +365,16 @@ ks_action_get (ctrl_t ctrl, uri_item_t keyservers,
                        || strcmp (uri->parsed_uri->scheme, "https") == 0);
       int is_ldap = 0;
 
-      if (ldap_only)
+      if ((ks_get_flags & (KS_GET_FLAG_ONLY_LDAP|KS_GET_FLAG_ONLY_AD)))
         is_hkp_s = is_http_s = 0;
 
 #if USE_LDAP
-      is_ldap = (strcmp (uri->parsed_uri->scheme, "ldap") == 0
-		 || strcmp (uri->parsed_uri->scheme, "ldaps") == 0
-		 || strcmp (uri->parsed_uri->scheme, "ldapi") == 0);
+      is_ldap = (!strcmp (uri->parsed_uri->scheme, "ldap")
+		 || !strcmp (uri->parsed_uri->scheme, "ldaps")
+		 || !strcmp (uri->parsed_uri->scheme, "ldapi")
+                 || uri->parsed_uri->opaque);
+#else
+      (void)newer;
 #endif
 
       if (is_hkp_s || is_http_s || is_ldap)
@@ -254,7 +384,8 @@ ks_action_get (ctrl_t ctrl, uri_item_t keyservers,
             {
 #if USE_LDAP
 	      if (is_ldap)
-		err = ks_ldap_get (ctrl, uri->parsed_uri, sl->d, &infp);
+		err = ks_ldap_get (ctrl, uri->parsed_uri, sl->d, ks_get_flags,
+                                   newer, &infp);
 	      else
 #endif
               if (is_hkp_s)
@@ -313,7 +444,7 @@ ks_action_fetch (ctrl_t ctrl, const char *url, estream_t outfp)
   if (!url)
     return gpg_error (GPG_ERR_INV_URI);
 
-  err = http_parse_uri (&parsed_uri, url, 1);
+  err = http_parse_uri (&parsed_uri, url, HTTP_PARSE_NO_SCHEME_CHECK);
   if (err)
     return err;
 
@@ -382,9 +513,10 @@ ks_action_put (ctrl_t ctrl, uri_item_t keyservers,
       int is_ldap = 0;
 
 #if USE_LDAP
-      is_ldap = (strcmp (uri->parsed_uri->scheme, "ldap") == 0
-		|| strcmp (uri->parsed_uri->scheme, "ldaps") == 0
-		|| strcmp (uri->parsed_uri->scheme, "ldapi") == 0);
+      is_ldap = (!strcmp (uri->parsed_uri->scheme, "ldap")
+                 || !strcmp (uri->parsed_uri->scheme, "ldaps")
+                 || !strcmp (uri->parsed_uri->scheme, "ldapi")
+                 || uri->parsed_uri->opaque);
 #endif
 
       if (is_http || is_ldap)
@@ -413,3 +545,94 @@ ks_action_put (ctrl_t ctrl, uri_item_t keyservers,
     err = first_err;
   return err;
 }
+
+
+
+/* Delete an OpenPGP key from all KEYSERVERS which use LDAP.  The key
+ * is specifified by PATTERNS.  */
+gpg_error_t
+ks_action_del (ctrl_t ctrl, uri_item_t keyservers, strlist_t patterns)
+{
+  gpg_error_t err = 0;
+  gpg_error_t first_err = 0;
+  int any_server = 0;
+  uri_item_t uri;
+
+  for (uri = keyservers; uri; uri = uri->next)
+    {
+#if USE_LDAP
+      if ( !strcmp (uri->parsed_uri->scheme, "ldap")
+           || !strcmp (uri->parsed_uri->scheme, "ldaps")
+           || !strcmp (uri->parsed_uri->scheme, "ldapi")
+           || uri->parsed_uri->opaque )
+        {
+          any_server = 1;
+          err = ks_ldap_del (ctrl, uri->parsed_uri, patterns);
+          if (err && !first_err)
+            first_err = err;
+        }
+#endif
+    }
+
+  if (!any_server)
+    err = gpg_error (GPG_ERR_NO_KEYSERVER); /* Actual: No LDAP keyserver */
+  else if (!err && first_err)
+    err = first_err;
+  return err;
+}
+
+
+/* Query the default LDAP server or the one given by URL using
+ * the filter expression FILTER.  Write the result to OUTFP.  */
+gpg_error_t
+ks_action_query (ctrl_t ctrl, const char *url, unsigned int ks_get_flags,
+                 const char *filter, char **attrs,
+                 gnupg_isotime_t newer, estream_t outfp)
+{
+#if USE_LDAP
+  gpg_error_t err;
+  estream_t infp = NULL;
+  uri_item_t puri;  /* The broken down URI (only one item used).  */
+
+  if (!url && (ks_get_flags & KS_GET_FLAG_ROOTDSE))
+    url = "ldap://";
+
+  err = ks_action_parse_uri (url, &puri);
+  if (err)
+    return err;
+
+  if ((ks_get_flags & KS_GET_FLAG_ROOTDSE))
+    {
+      /* Reset authentication for a serverless connection.  */
+      puri->parsed_uri->ad_current = 0;
+      puri->parsed_uri->auth = NULL;
+    }
+
+  if (!strcmp (puri->parsed_uri->scheme, "ldap")
+      || !strcmp (puri->parsed_uri->scheme, "ldaps")
+      || !strcmp (puri->parsed_uri->scheme, "ldapi")
+      || puri->parsed_uri->opaque)
+    {
+      err = ks_ldap_query (ctrl, puri->parsed_uri, ks_get_flags, filter,
+                           attrs, newer, &infp);
+      if (!err)
+        err = copy_stream (infp, outfp);
+    }
+  else
+    err = gpg_error (GPG_ERR_CONFIGURATION); /* No LDAP server known.  */
+
+  es_fclose (infp);
+  release_uri_item_list (puri);
+  return err;
+
+#else /* !USE_LDAP */
+  (void)ctrl;
+  (void)url;
+  (void)ks_get_flags;
+  (void)filter;
+  (void)attrs;
+  (void)newer;
+  (void)outfp;
+  return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+#endif
+}
